package histogram;

/**
 * Utilidades para la obtención de datos de tipo histograma.
 * 
 * <p>Proporciona métodos que permiten obtener cuentas o porcentajes dentro de los bins correspondientes a un array de fronteras.
 * <p>El tipo <code>BASIC</code> considera el espacio (-inf, inf), mientras que el tipo <code>MODULE</code> se aplica a valores en módulo y por tanto a [0, inf)
 *
 * <p>Ejemplo1: Para una secuencia de fronteras {-2, -1, -0.5, 0, 10 } permite obtener cuentas o porcentajes dentro de los bins
 * (-inf,-2] (-1,-0.5] (-0.5,0] (0,10] (10,inf)
 * 
 * <p>Ejemplo2: Para una secuencia de fronteras {0.5, 2, 10 } permite obtener cuentas o porcentajes dentro de los bins
 * (-inf,0.5] (0.5,2] (2,10] (10,inf) si se usa el tipo <code>BASIC</code>, o [0,0.5] (0.5,2] (2,10] (10,inf) para el tipo <code>MODULE</code>.
 * 
 * 
 */
public class BinCounter {
    
    /**
     * Enumeración de los cuatro tipos de conteos permitidos.
     * <ul>
     * <li> BASIC.- Cada bin lleva la cuenta de los valores que le corresponden estrictamente.</li>
     * <li> BASIC_CUMULATIVE_LR.- Idem BASIC, pero cada bin suma además la cuenta del bin anterior.</li>
     * <li> BASIC_CUMULATIVE_RL.- Idem BASIC, pero cada bin suma además la cuenta del bin siguiente.</li>
     * <li> MODULE.- Cada bin lleva la cuenta de los valores que le corresponden una vez se les practica el módulo (|x|).</li>
     * <li> MODULE_CUMULATIVE_LR.- Idem MODULE, pero cada bin suma además la cuenta del bin anterior.</li>
     * <li> MODULE_CUMULATIVE_RL.- Idem MODULE, pero cada bin suma además la cuenta del bin siguiente.</li>
     * </ul>
     * 
     * 
     */
    public static enum Type { BASIC, BASIC_CUMULATIVE_LR, BASIC_CUMULATIVE_RL, MODULE, MODULE_CUMULATIVE_LR, MODULE_CUMULATIVE_RL;
        public boolean isBasic() { return this==BASIC || this==BASIC_CUMULATIVE_LR || this==BASIC_CUMULATIVE_RL;}
        public boolean isLRCumulative() { return this==BASIC_CUMULATIVE_LR || this==MODULE_CUMULATIVE_LR ;}
        public boolean isRLCumulative() { return this==BASIC_CUMULATIVE_RL || this==MODULE_CUMULATIVE_RL ;}
    }
    

    /**
     * Un histograma definido por su tipo y la lista de fronteras de los bins.
     * @param type el tipo de histograma conforme a la enumeración interna
     */
	//AQUI EL CONSTRUCTOR    

    /**
     * Dado un array de datos realiza los conteos para cada bin
     * @param bins la secuencia de fronteras de los bins
     * @param datos el array de datos
     * @throws IllegalArgumentException Si el array de fronteras no es monótono creciente o contiene valores negativos para un histograma de módulos.
     */
      public void calculateBinCounts(double[] bins, double[] datos) {
    }
    
    /**
     * Dado un array de conteos, proporciona los correspondientes porcentajes (tiene en cuenta el tipo de histograma).
     * Tiene en cuenta el tipo de histograma (si es acumulativo o no) para proporcionar porcentajes del mismo tipo.
     * @return el array de porcentajes.
     */
    public double[] calculateFreqs() {
    }

    /**
     * Representa textualmente el array de conteos con uno por fila precedido de la información del bin
     * @return el texto con las filas formadas por bin\tconteo\n
     */
    public String countsToText(){
        StringBuilder sb=new StringBuilder(String.format("%s,%.2f]\t%d\n",(type.isBasic()?"(-inf":"[0.0"),bins[0],binCounts[0]));
        for (int i=1; i<bins.length;i++)
            sb.append(String.format("(%.2f,%.2f]\t%d\n",bins[i-1],bins[i],binCounts[i]));
       sb.append(String.format("(%.2f,inf)\t%d\n",bins[bins.length-1],binCounts[bins.length]));
    return sb.toString();
    }
    
    /**
     * Representa textualmente el array de porcentajes con uno por fila precedido de la información del bin
     * @return el texto con las filas formadas por bin\porcentaje\n
     */
    public String freqsToText(){
        double[] freqs=calculateFreqs();
        StringBuilder sb=new StringBuilder(String.format("%s,%.2f]\t%.2f\n",(type.isBasic()?"(-inf":"[0.0"),bins[0],freqs[0]));
        for (int i=1; i<bins.length;i++)
            sb.append(String.format("(%.2f,%.2f]\t%.2f\n",bins[i-1],bins[i],freqs[i]));
       sb.append(String.format("(%.2f,inf)\t%.2f\n",bins[bins.length-1],freqs[bins.length]));
    return sb.toString();
    }
  
}
